@@ -1,5 +1,8 @@
+import json
+import os
 from pathlib import Path
 
+from manubot.ai_editor import env_vars
 from manubot.ai_editor.models import ManuscriptRevisionModel
 from manubot.ai_editor.utils import get_yaml_field, SENTENCE_END_PATTERN
 
@@ -21,10 +24,6 @@ class ManuscriptEditor:
         self.title = get_yaml_field(metadata_file, "title")
         self.keywords = get_yaml_field(metadata_file, "keywords")
 
-    @staticmethod
-    def line_is_not_part_of_paragraph(line: str) -> bool:
-        return line.startswith(("#", "<!--")) or line.strip() == ""
-
     @staticmethod
     def revise_and_write_paragraph(
         paragraph: list[str],
@@ -47,40 +46,96 @@ class ManuscriptEditor:
         """
         # Process the paragraph and revise it with model
         paragraph_text = " ".join(paragraph)
-        paragraph_revised = revision_model.revise_paragraph(
-            paragraph_text, section_name
-        )
 
-        # put sentences into new lines
-        paragraph_revised = SENTENCE_END_PATTERN.sub(".\n", paragraph_revised)
-        paragraph_revised = paragraph_revised
+        error_message = None
+        try:
+            paragraph_revised = revision_model.revise_paragraph(
+                paragraph_text,
+                section_name,
+            )
+
+            if paragraph_revised.strip() == "":
+                raise Exception("The AI model returned an empty string ('')")
+        except Exception as e:
+            error_message = f"""
+<!--
+ERROR: this paragraph could not be revised with the AI model due to the following error:
+
+{str(e)}
+-->
+            """.strip()
+
+        if error_message is not None:
+            paragraph_revised = (
+                error_message
+                + "\n"
+                + ManuscriptEditor.convert_sentence_ends_to_newlines(paragraph_text)
+            )
+        else:
+            # put sentences into new lines
+            paragraph_revised = ManuscriptEditor.convert_sentence_ends_to_newlines(
+                paragraph_revised
+            )
 
         if outfile is not None:
             outfile.write(paragraph_revised + "\n")
         else:
             return paragraph_text, paragraph_revised
 
-    def get_section_from_filename(self, filename: str) -> str:
+    @staticmethod
+    def convert_sentence_ends_to_newlines(paragraph: str) -> str:
+        """
+        Converts sentence ends to newlines.
+
+        Args:
+            paragraph: paragraph to convert.
+
+        Returns:
+            Converted paragraph.
+        """
+        return SENTENCE_END_PATTERN.sub(r".\n\1", paragraph)
+
+    @staticmethod
+    def get_section_from_filename(filename: str) -> str | None:
         """
         Returns the section name of a file based on its filename.
         """
         filename = filename.lower()
 
+        if env_vars.SECTIONS_MAPPING in os.environ:
+            sections_mapping = os.environ[env_vars.SECTIONS_MAPPING]
+            try:
+                sections_mapping = json.loads(sections_mapping)
+                if filename in sections_mapping:
+                    return sections_mapping[filename]
+            except json.JSONDecodeError:
+                print(
+                    f"Invalid JSON in environment variable {env_vars.SECTIONS_MAPPING}",
+                    flush=True,
+                )
+
         if "abstract" in filename:
             return "abstract"
         elif "introduction" in filename:
             return "introduction"
-        elif "results" in filename:
+        elif "result" in filename:
             return "results"
         elif "discussion" in filename:
             return "discussion"
-        elif "methods" in filename:
+        elif "conclusion" in filename:
+            return "conclusions"
+        elif "method" in filename:
             return "methods"
         elif "supplementary" in filename:
             return "supplementary_material"
         else:
             return None
 
+    @staticmethod
+    def line_is_not_part_of_paragraph(line: str, include_blank=True) -> bool:
+        prefixes = ("![", "|", "<!--", "$$", "#", "```")
+        return line.startswith(prefixes) or (include_blank and line.strip() == "")
+
     def revise_file(
         self,
         input_filename: str,
@@ -116,7 +171,7 @@ class ManuscriptEditor:
             for line in infile:
                 # if line is starting either an "image paragraph", a "table paragraph" or a "html comment paragraph",
                 # then skip all lines until the end of that paragraph
-                if line.startswith(("![", "|", "<!--", "$$")):
+                if self.line_is_not_part_of_paragraph(line, include_blank=False):
                     while line is not None and line.strip() != "":
                         outfile.write(line)
                         line = next(infile, None)
@@ -125,9 +180,9 @@ class ManuscriptEditor:
                 if line is None:
                     break
 
-                # if the line is a comment or a section name, write it directly
-                # to the output file
-                if self.line_is_not_part_of_paragraph(line) and len(paragraph) == 0:
+                # if the line is empty and we didn't start a paragraph yet,
+                # write it directly to the output file
+                if line.strip() == "" and len(paragraph) == 0:
                     outfile.write(line)
 
                 # If the line is blank, it indicates the end of a paragraph
